@@ -8,17 +8,23 @@ from datetime import (
     tzinfo,
 )
 import functools
-from typing import Any
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Final,
+    Generator,
+    cast,
+)
 
 from dateutil.relativedelta import relativedelta
-import matplotlib.dates as dates
+import matplotlib.dates as mdates
 from matplotlib.ticker import (
     AutoLocator,
     Formatter,
     Locator,
 )
 from matplotlib.transforms import nonsingular
-import matplotlib.units as units
+import matplotlib.units as munits
 import numpy as np
 
 from pandas._libs import lib
@@ -27,7 +33,7 @@ from pandas._libs.tslibs import (
     to_offset,
 )
 from pandas._libs.tslibs.dtypes import FreqGroup
-from pandas._libs.tslibs.offsets import BaseOffset
+from pandas._typing import F
 
 from pandas.core.dtypes.common import (
     is_float,
@@ -51,15 +57,18 @@ from pandas.core.indexes.period import (
 )
 import pandas.core.tools.datetimes as tools
 
+if TYPE_CHECKING:
+    from pandas._libs.tslibs.offsets import BaseOffset
+
 # constants
-HOURS_PER_DAY = 24.0
-MIN_PER_HOUR = 60.0
-SEC_PER_MIN = 60.0
+HOURS_PER_DAY: Final = 24.0
+MIN_PER_HOUR: Final = 60.0
+SEC_PER_MIN: Final = 60.0
 
-SEC_PER_HOUR = SEC_PER_MIN * MIN_PER_HOUR
-SEC_PER_DAY = SEC_PER_HOUR * HOURS_PER_DAY
+SEC_PER_HOUR: Final = SEC_PER_MIN * MIN_PER_HOUR
+SEC_PER_DAY: Final = SEC_PER_HOUR * HOURS_PER_DAY
 
-MUSEC_PER_DAY = 10 ** 6 * SEC_PER_DAY
+MUSEC_PER_DAY: Final = 10**6 * SEC_PER_DAY
 
 _mpl_units = {}  # Cache for units overwritten by us
 
@@ -76,7 +85,7 @@ def get_pairs():
     return pairs
 
 
-def register_pandas_matplotlib_converters(func):
+def register_pandas_matplotlib_converters(func: F) -> F:
     """
     Decorator applying pandas_converters.
     """
@@ -86,11 +95,11 @@ def register_pandas_matplotlib_converters(func):
         with pandas_converters():
             return func(*args, **kwargs)
 
-    return wrapper
+    return cast(F, wrapper)
 
 
 @contextlib.contextmanager
-def pandas_converters():
+def pandas_converters() -> Generator[None, None, None]:
     """
     Context manager registering pandas' converters for a plot.
 
@@ -111,48 +120,46 @@ def pandas_converters():
             deregister()
 
 
-def register():
+def register() -> None:
     pairs = get_pairs()
     for type_, cls in pairs:
         # Cache previous converter if present
-        if type_ in units.registry and not isinstance(units.registry[type_], cls):
-            previous = units.registry[type_]
+        if type_ in munits.registry and not isinstance(munits.registry[type_], cls):
+            previous = munits.registry[type_]
             _mpl_units[type_] = previous
         # Replace with pandas converter
-        units.registry[type_] = cls()
+        munits.registry[type_] = cls()
 
 
-def deregister():
+def deregister() -> None:
     # Renamed in pandas.plotting.__init__
     for type_, cls in get_pairs():
         # We use type to catch our classes directly, no inheritance
-        if type(units.registry.get(type_)) is cls:
-            units.registry.pop(type_)
+        if type(munits.registry.get(type_)) is cls:
+            munits.registry.pop(type_)
 
     # restore the old keys
     for unit, formatter in _mpl_units.items():
         if type(formatter) not in {DatetimeConverter, PeriodConverter, TimeConverter}:
             # make it idempotent by excluding ours.
-            units.registry[unit] = formatter
+            munits.registry[unit] = formatter
 
 
 def _to_ordinalf(tm: pydt.time) -> float:
-    tot_sec = tm.hour * 3600 + tm.minute * 60 + tm.second + tm.microsecond / 10 ** 6
+    tot_sec = tm.hour * 3600 + tm.minute * 60 + tm.second + tm.microsecond / 10**6
     return tot_sec
 
 
 def time2num(d):
     if isinstance(d, str):
-        parsed = tools.to_datetime(d)
-        if not isinstance(parsed, datetime):
-            raise ValueError(f"Could not parse time {d}")
+        parsed = Timestamp(d)
         return _to_ordinalf(parsed.time())
     if isinstance(d, pydt.time):
         return _to_ordinalf(d)
     return d
 
 
-class TimeConverter(units.ConversionInterface):
+class TimeConverter(munits.ConversionInterface):
     @staticmethod
     def convert(value, unit, axis):
         valid_types = (str, pydt.time)
@@ -165,13 +172,13 @@ class TimeConverter(units.ConversionInterface):
         return value
 
     @staticmethod
-    def axisinfo(unit, axis) -> units.AxisInfo | None:
+    def axisinfo(unit, axis) -> munits.AxisInfo | None:
         if unit != "time":
             return None
 
         majloc = AutoLocator()
         majfmt = TimeFormatter(majloc)
-        return units.AxisInfo(majloc=majloc, majfmt=majfmt, label="time")
+        return munits.AxisInfo(majloc=majloc, majfmt=majfmt, label="time")
 
     @staticmethod
     def default_units(x, axis) -> str:
@@ -180,10 +187,10 @@ class TimeConverter(units.ConversionInterface):
 
 # time formatter
 class TimeFormatter(Formatter):
-    def __init__(self, locs):
+    def __init__(self, locs) -> None:
         self.locs = locs
 
-    def __call__(self, x, pos=0) -> str:
+    def __call__(self, x, pos: int = 0) -> str:
         """
         Return the time of day as a formatted string.
 
@@ -203,7 +210,7 @@ class TimeFormatter(Formatter):
         """
         fmt = "%H:%M:%S.%f"
         s = int(x)
-        msus = round((x - s) * 10 ** 6)
+        msus = round((x - s) * 10**6)
         ms = msus // 1000
         us = msus % 1000
         m, s = divmod(s, 60)
@@ -222,7 +229,7 @@ class TimeFormatter(Formatter):
 # Period Conversion
 
 
-class PeriodConverter(dates.DateConverter):
+class PeriodConverter(mdates.DateConverter):
     @staticmethod
     def convert(values, units, axis):
         if is_nested_list_like(values):
@@ -268,7 +275,7 @@ def get_datevalue(date, freq):
 
 
 # Datetime Conversion
-class DatetimeConverter(dates.DateConverter):
+class DatetimeConverter(mdates.DateConverter):
     @staticmethod
     def convert(values, unit, axis):
         # values might be a 1-d array, or a list-like of arrays.
@@ -282,12 +289,12 @@ class DatetimeConverter(dates.DateConverter):
     def _convert_1d(values, unit, axis):
         def try_parse(values):
             try:
-                return dates.date2num(tools.to_datetime(values))
+                return mdates.date2num(tools.to_datetime(values))
             except Exception:
                 return values
 
         if isinstance(values, (datetime, pydt.date, np.datetime64, pydt.time)):
-            return dates.date2num(values)
+            return mdates.date2num(values)
         elif is_integer(values) or is_float(values):
             return values
         elif isinstance(values, str):
@@ -310,12 +317,12 @@ class DatetimeConverter(dates.DateConverter):
             except Exception:
                 pass
 
-            values = dates.date2num(values)
+            values = mdates.date2num(values)
 
         return values
 
     @staticmethod
-    def axisinfo(unit: tzinfo | None, axis) -> units.AxisInfo:
+    def axisinfo(unit: tzinfo | None, axis) -> munits.AxisInfo:
         """
         Return the :class:`~matplotlib.units.AxisInfo` for *unit*.
 
@@ -329,17 +336,17 @@ class DatetimeConverter(dates.DateConverter):
         datemin = pydt.date(2000, 1, 1)
         datemax = pydt.date(2010, 1, 1)
 
-        return units.AxisInfo(
+        return munits.AxisInfo(
             majloc=majloc, majfmt=majfmt, label="", default_limits=(datemin, datemax)
         )
 
 
-class PandasAutoDateFormatter(dates.AutoDateFormatter):
-    def __init__(self, locator, tz=None, defaultfmt="%Y-%m-%d"):
-        dates.AutoDateFormatter.__init__(self, locator, tz, defaultfmt)
+class PandasAutoDateFormatter(mdates.AutoDateFormatter):
+    def __init__(self, locator, tz=None, defaultfmt: str = "%Y-%m-%d") -> None:
+        mdates.AutoDateFormatter.__init__(self, locator, tz, defaultfmt)
 
 
-class PandasAutoDateLocator(dates.AutoDateLocator):
+class PandasAutoDateLocator(mdates.AutoDateLocator):
     def get_locator(self, dmin, dmax):
         """Pick the best locator based on a distance."""
         delta = relativedelta(dmax, dmin)
@@ -357,18 +364,17 @@ class PandasAutoDateLocator(dates.AutoDateLocator):
             locator.axis.set_data_interval(*self.axis.get_data_interval())
             return locator
 
-        return dates.AutoDateLocator.get_locator(self, dmin, dmax)
+        return mdates.AutoDateLocator.get_locator(self, dmin, dmax)
 
     def _get_unit(self):
         return MilliSecondLocator.get_unit_generic(self._freq)
 
 
-class MilliSecondLocator(dates.DateLocator):
-
+class MilliSecondLocator(mdates.DateLocator):
     UNIT = 1.0 / (24 * 3600 * 1000)
 
-    def __init__(self, tz):
-        dates.DateLocator.__init__(self, tz)
+    def __init__(self, tz) -> None:
+        mdates.DateLocator.__init__(self, tz)
         self._interval = 1.0
 
     def _get_unit(self):
@@ -376,7 +382,7 @@ class MilliSecondLocator(dates.DateLocator):
 
     @staticmethod
     def get_unit_generic(freq):
-        unit = dates.RRuleLocator.get_unit_generic(freq)
+        unit = mdates.RRuleLocator.get_unit_generic(freq)
         if unit < 0:
             return MilliSecondLocator.UNIT
         return unit
@@ -389,7 +395,7 @@ class MilliSecondLocator(dates.DateLocator):
             return []
 
         # We need to cap at the endpoints of valid datetime
-        nmax, nmin = dates.date2num((dmax, dmin))
+        nmax, nmin = mdates.date2num((dmax, dmin))
 
         num = (nmax - nmin) * 86400 * 1000
         max_millis_ticks = 6
@@ -397,9 +403,8 @@ class MilliSecondLocator(dates.DateLocator):
             if num <= interval * (max_millis_ticks - 1):
                 self._interval = interval
                 break
-            else:
-                # We went through the whole loop without breaking, default to 1
-                self._interval = 1000.0
+            # We went through the whole loop without breaking, default to 1
+            self._interval = 1000.0
 
         estimate = (nmax - nmin) / (self._get_unit() * self._get_interval())
 
@@ -419,12 +424,12 @@ class MilliSecondLocator(dates.DateLocator):
 
         try:
             if len(all_dates) > 0:
-                locs = self.raise_if_exceeds(dates.date2num(all_dates))
+                locs = self.raise_if_exceeds(mdates.date2num(all_dates))
                 return locs
         except Exception:  # pragma: no cover
             pass
 
-        lims = dates.date2num([dmin, dmax])
+        lims = mdates.date2num([dmin, dmax])
         return lims
 
     def _get_interval(self):
@@ -437,8 +442,8 @@ class MilliSecondLocator(dates.DateLocator):
         # We need to cap at the endpoints of valid datetime
         dmin, dmax = self.datalim_to_dt()
 
-        vmin = dates.date2num(dmin)
-        vmax = dates.date2num(dmax)
+        vmin = mdates.date2num(dmin)
+        vmax = mdates.date2num(dmax)
 
         return self.nonsingular(vmin, vmax)
 
@@ -527,35 +532,37 @@ def has_level_label(label_flags: np.ndarray, vmin: float) -> bool:
 
 
 def _daily_finder(vmin, vmax, freq: BaseOffset):
-    dtype_code = freq._period_dtype_code
+    # error: "BaseOffset" has no attribute "_period_dtype_code"
+    dtype_code = freq._period_dtype_code  # type: ignore[attr-defined]
+    freq_group = FreqGroup.from_period_dtype_code(dtype_code)
 
     periodsperday = -1
 
     if dtype_code >= FreqGroup.FR_HR.value:
-        if dtype_code == FreqGroup.FR_NS.value:
+        if freq_group == FreqGroup.FR_NS:
             periodsperday = 24 * 60 * 60 * 1000000000
-        elif dtype_code == FreqGroup.FR_US.value:
+        elif freq_group == FreqGroup.FR_US:
             periodsperday = 24 * 60 * 60 * 1000000
-        elif dtype_code == FreqGroup.FR_MS.value:
+        elif freq_group == FreqGroup.FR_MS:
             periodsperday = 24 * 60 * 60 * 1000
-        elif dtype_code == FreqGroup.FR_SEC.value:
+        elif freq_group == FreqGroup.FR_SEC:
             periodsperday = 24 * 60 * 60
-        elif dtype_code == FreqGroup.FR_MIN.value:
+        elif freq_group == FreqGroup.FR_MIN:
             periodsperday = 24 * 60
-        elif dtype_code == FreqGroup.FR_HR.value:
+        elif freq_group == FreqGroup.FR_HR:
             periodsperday = 24
         else:  # pragma: no cover
             raise ValueError(f"unexpected frequency: {dtype_code}")
         periodsperyear = 365 * periodsperday
         periodspermonth = 28 * periodsperday
 
-    elif dtype_code == FreqGroup.FR_BUS.value:
+    elif freq_group == FreqGroup.FR_BUS:
         periodsperyear = 261
         periodspermonth = 19
-    elif dtype_code == FreqGroup.FR_DAY.value:
+    elif freq_group == FreqGroup.FR_DAY:
         periodsperyear = 365
         periodspermonth = 28
-    elif FreqGroup.get_freq_group(dtype_code) == FreqGroup.FR_WK:
+    elif freq_group == FreqGroup.FR_WK:
         periodsperyear = 52
         periodspermonth = 3
     else:  # pragma: no cover
@@ -568,6 +575,8 @@ def _daily_finder(vmin, vmax, freq: BaseOffset):
         Period(ordinal=int(vmin), freq=freq),
         Period(ordinal=int(vmax), freq=freq),
     )
+    assert isinstance(vmin, Period)
+    assert isinstance(vmax, Period)
     span = vmax.ordinal - vmin.ordinal + 1
     dates_ = period_range(start=vmin, end=vmax, freq=freq)
     # Initialize the output
@@ -593,7 +602,7 @@ def _daily_finder(vmin, vmax, freq: BaseOffset):
         day_start = period_break(dates_, "day")
         month_start = period_break(dates_, "month")
 
-        def _hour_finder(label_interval, force_year_start):
+        def _hour_finder(label_interval, force_year_start) -> None:
             _hour = dates_.hour
             _prev_hour = (dates_ - 1 * dates_.freq).hour
             hour_start = (_hour - _prev_hour) != 0
@@ -606,7 +615,7 @@ def _daily_finder(vmin, vmax, freq: BaseOffset):
             if force_year_start and not has_level_label(year_start, vmin_orig):
                 info_fmt[first_label(day_start)] = "%H:%M\n%d-%b\n%Y"
 
-        def _minute_finder(label_interval):
+        def _minute_finder(label_interval) -> None:
             hour_start = period_break(dates_, "hour")
             _minute = dates_.minute
             _prev_minute = (dates_ - 1 * dates_.freq).minute
@@ -619,7 +628,7 @@ def _daily_finder(vmin, vmax, freq: BaseOffset):
             info_fmt[day_start] = "%H:%M\n%d-%b"
             info_fmt[year_start] = "%H:%M\n%d-%b\n%Y"
 
-        def _second_finder(label_interval):
+        def _second_finder(label_interval) -> None:
             minute_start = period_break(dates_, "minute")
             _second = dates_.second
             _prev_second = (dates_ - 1 * dates_.freq).second
@@ -859,7 +868,8 @@ def _quarterly_finder(vmin, vmax, freq):
         info_fmt[year_start] = "%F"
 
     else:
-        years = dates_[year_start] // 4 + 1
+        # https://github.com/pandas-dev/pandas/pull/47602
+        years = dates_[year_start] // 4 + 1970
         nyears = span / periodsperyear
         (min_anndef, maj_anndef) = _get_default_annual_spacing(nyears)
         major_idx = year_start[(years % maj_anndef == 0)]
@@ -891,15 +901,15 @@ def _annual_finder(vmin, vmax, freq):
 
 
 def get_finder(freq: BaseOffset):
-    dtype_code = freq._period_dtype_code
-    fgroup = (dtype_code // 1000) * 1000
-    fgroup = FreqGroup(fgroup)
+    # error: "BaseOffset" has no attribute "_period_dtype_code"
+    dtype_code = freq._period_dtype_code  # type: ignore[attr-defined]
+    fgroup = FreqGroup.from_period_dtype_code(dtype_code)
 
     if fgroup == FreqGroup.FR_ANN:
         return _annual_finder
     elif fgroup == FreqGroup.FR_QTR:
         return _quarterly_finder
-    elif dtype_code == FreqGroup.FR_MTH.value:
+    elif fgroup == FreqGroup.FR_MTH:
         return _monthly_finder
     elif (dtype_code >= FreqGroup.FR_BUS.value) or fgroup == FreqGroup.FR_WK:
         return _daily_finder
@@ -913,7 +923,7 @@ class TimeSeries_DateLocator(Locator):
 
     Parameters
     ----------
-    freq : {var}
+    freq : BaseOffset
         Valid frequency specifier.
     minor_locator : {False, True}, optional
         Whether the locator is for minor ticks (True) or not.
@@ -927,15 +937,15 @@ class TimeSeries_DateLocator(Locator):
 
     def __init__(
         self,
-        freq,
-        minor_locator=False,
-        dynamic_mode=True,
-        base=1,
-        quarter=1,
-        month=1,
-        day=1,
+        freq: BaseOffset,
+        minor_locator: bool = False,
+        dynamic_mode: bool = True,
+        base: int = 1,
+        quarter: int = 1,
+        month: int = 1,
+        day: int = 1,
         plot_obj=None,
-    ):
+    ) -> None:
         freq = to_offset(freq)
         self.freq = freq
         self.base = base
@@ -1004,7 +1014,7 @@ class TimeSeries_DateFormatter(Formatter):
 
     Parameters
     ----------
-    freq : {int, string}
+    freq : BaseOffset
         Valid frequency specifier.
     minor_locator : bool, default False
         Whether the current formatter should apply to minor ticks (True) or
@@ -1015,11 +1025,11 @@ class TimeSeries_DateFormatter(Formatter):
 
     def __init__(
         self,
-        freq,
+        freq: BaseOffset,
         minor_locator: bool = False,
         dynamic_mode: bool = True,
         plot_obj=None,
-    ):
+    ) -> None:
         freq = to_offset(freq)
         self.format = None
         self.freq = freq
@@ -1044,7 +1054,7 @@ class TimeSeries_DateFormatter(Formatter):
         self.formatdict = {x: f for (x, _, _, f) in format}
         return self.formatdict
 
-    def set_locs(self, locs):
+    def set_locs(self, locs) -> None:
         """Sets the locations of the ticks"""
         # don't actually use the locs. This is just needed to work with
         # matplotlib. Force to use vmin, vmax
@@ -1059,15 +1069,16 @@ class TimeSeries_DateFormatter(Formatter):
             (vmin, vmax) = (vmax, vmin)
         self._set_default_format(vmin, vmax)
 
-    def __call__(self, x, pos=0) -> str:
-
+    def __call__(self, x, pos: int = 0) -> str:
         if self.formatdict is None:
             return ""
         else:
             fmt = self.formatdict.pop(x, "")
             if isinstance(fmt, np.bytes_):
                 fmt = fmt.decode("utf-8")
-            return Period(ordinal=int(x), freq=self.freq).strftime(fmt)
+            period = Period(ordinal=int(x), freq=self.freq)
+            assert isinstance(period, Period)
+            return period.strftime(fmt)
 
 
 class TimeSeries_TimedeltaFormatter(Formatter):
@@ -1080,7 +1091,7 @@ class TimeSeries_TimedeltaFormatter(Formatter):
         """
         Convert seconds to 'D days HH:MM:SS.F'
         """
-        s, ns = divmod(x, 10 ** 9)
+        s, ns = divmod(x, 10**9)
         m, s = divmod(s, 60)
         h, m = divmod(m, 60)
         d, h = divmod(h, 24)
@@ -1092,9 +1103,7 @@ class TimeSeries_TimedeltaFormatter(Formatter):
             s = f"{int(d):d} days {s}"
         return s
 
-    def __call__(self, x, pos=0) -> str:
+    def __call__(self, x, pos: int = 0) -> str:
         (vmin, vmax) = tuple(self.axis.get_view_interval())
-        n_decimals = int(np.ceil(np.log10(100 * 10 ** 9 / abs(vmax - vmin))))
-        if n_decimals > 9:
-            n_decimals = 9
+        n_decimals = min(int(np.ceil(np.log10(100 * 10**9 / abs(vmax - vmin)))), 9)
         return self.format_timedelta_ticks(x, pos, n_decimals)
