@@ -43,6 +43,12 @@ import markdown
 import requests
 import yaml
 
+api_token = os.environ.get("GITHUB_TOKEN")
+if api_token is not None:
+    GITHUB_API_HEADERS = {"Authorization": f"Bearer {api_token}"}
+else:
+    GITHUB_API_HEADERS = {}
+
 
 class Preprocessors:
     """
@@ -168,7 +174,9 @@ class Preprocessors:
         for user in (
             context["maintainers"]["active"] + context["maintainers"]["inactive"]
         ):
-            resp = requests.get(f"https://api.github.com/users/{user}")
+            resp = requests.get(
+                f"https://api.github.com/users/{user}", headers=GITHUB_API_HEADERS
+            )
             if resp.status_code == 403:
                 sys.stderr.write(
                     "WARN: GitHub API quota exceeded when fetching maintainers\n"
@@ -199,7 +207,10 @@ class Preprocessors:
         context["releases"] = []
 
         github_repo_url = context["main"]["github_repo_url"]
-        resp = requests.get(f"https://api.github.com/repos/{github_repo_url}/releases")
+        resp = requests.get(
+            f"https://api.github.com/repos/{github_repo_url}/releases",
+            headers=GITHUB_API_HEADERS,
+        )
         if resp.status_code == 403:
             sys.stderr.write("WARN: GitHub API quota exceeded when fetching releases\n")
             resp_bkp = requests.get(context["main"]["production_url"] + "releases.json")
@@ -267,7 +278,7 @@ class Preprocessors:
             context["pdeps"][status].append(
                 {
                     "title": title,
-                    "url": f"/pdeps/{html_file}",
+                    "url": f"pdeps/{html_file}",
                 }
             )
 
@@ -275,7 +286,8 @@ class Preprocessors:
         github_repo_url = context["main"]["github_repo_url"]
         resp = requests.get(
             "https://api.github.com/search/issues?"
-            f"q=is:pr is:open label:PDEP repo:{github_repo_url}"
+            f"q=is:pr is:open label:PDEP repo:{github_repo_url}",
+            headers=GITHUB_API_HEADERS,
         )
         if resp.status_code == 403:
             sys.stderr.write("WARN: GitHub API quota exceeded when fetching pdeps\n")
@@ -371,13 +383,12 @@ def extend_base_template(content: str, base_template: str) -> str:
 def main(
     source_path: str,
     target_path: str,
-    base_url: str,
 ) -> int:
     """
     Copy every file in the source directory to the target directory.
 
     For ``.md`` and ``.html`` files, render them with the context
-    before copyings them. ``.md`` files are transformed to HTML.
+    before copying them. ``.md`` files are transformed to HTML.
     """
     config_fname = os.path.join(source_path, "config.yml")
 
@@ -385,7 +396,7 @@ def main(
     os.makedirs(target_path, exist_ok=True)
 
     sys.stderr.write("Generating context...\n")
-    context = get_context(config_fname, base_url=base_url, target_path=target_path)
+    context = get_context(config_fname, target_path=target_path)
     sys.stderr.write("Context generated\n")
 
     templates_path = os.path.join(source_path, context["main"]["templates_path"])
@@ -408,6 +419,7 @@ def main(
                     content, extensions=context["main"]["markdown_extensions"]
                 )
                 content = extend_base_template(body, context["main"]["base_template"])
+            context["base_url"] = "".join(["../"] * os.path.normpath(fname).count("/"))
             content = jinja_env.from_string(content).render(**context)
             fname = os.path.splitext(fname)[0] + ".html"
             with open(os.path.join(target_path, fname), "w") as f:
@@ -426,8 +438,5 @@ if __name__ == "__main__":
     parser.add_argument(
         "--target-path", default="build", help="directory where to write the output"
     )
-    parser.add_argument(
-        "--base-url", default="", help="base url where the website is served from"
-    )
     args = parser.parse_args()
-    sys.exit(main(args.source_path, args.target_path, args.base_url))
+    sys.exit(main(args.source_path, args.target_path))
